@@ -33,6 +33,8 @@
 #' @param .scale Must be strictly positive.
 #' @param .rate An alternative way to specify the `.scale`
 #' @param .num_sims The number of randomly generated simulations you want.
+#' @param .return_tibble A logical value indicating whether to return the result
+#' as a tibble. Default is TRUE.
 #'
 #' @examples
 #' tidy_paralogistic()
@@ -40,11 +42,14 @@
 #' @return
 #' A tibble of randomly generated data.
 #'
+#' @name tidy_paralogistic
+NULL
+
 #' @export
-#'
+#' @rdname tidy_paralogistic
 
 tidy_paralogistic <- function(.n = 50, .shape = 1, .rate = 1, .scale = 1 / .rate,
-                              .num_sims = 1) {
+                              .num_sims = 1, .return_tibble = TRUE) {
 
   # Tidyeval ----
   n <- as.integer(.n)
@@ -52,6 +57,7 @@ tidy_paralogistic <- function(.n = 50, .shape = 1, .rate = 1, .scale = 1 / .rate
   shape <- as.numeric(.shape)
   scale <- as.numeric(.scale)
   rate <- as.numeric(.rate)
+  ret_tbl <- as.logical(.return_tibble)
 
   # Checks ----
   if (!is.integer(n) | n < 0) {
@@ -86,27 +92,28 @@ tidy_paralogistic <- function(.n = 50, .shape = 1, .rate = 1, .scale = 1 / .rate
   qs <- seq(0, 1, (1 / (n - 1)))
   ps <- qs
 
-  df <- dplyr::tibble(sim_number = as.factor(x)) %>%
-    dplyr::group_by(sim_number) %>%
-    dplyr::mutate(x = list(1:n)) %>%
-    dplyr::mutate(y = list(actuar::rparalogis(
-      n = n, shape = shape, rate = rate,
-      scale = scale
-    ))) %>%
-    dplyr::mutate(d = list(density(unlist(y), n = n)[c("x", "y")] %>%
-      purrr::set_names("dx", "dy") %>%
-      dplyr::as_tibble())) %>%
-    dplyr::mutate(p = list(actuar::pparalogis(unlist(y),
-      shape = shape, rate = rate,
-      scale = scale
-    ))) %>%
-    dplyr::mutate(q = list(actuar::qparalogis(unlist(p),
-      shape = shape, rate = rate,
-      scale = scale
-    ))) %>%
-    tidyr::unnest(cols = c(x, y, d, p, q)) %>%
-    dplyr::ungroup()
+  # Create a data.table with one row per simulation
+  df <- data.table::CJ(sim_number = factor(1:num_sims), x = 1:n)
 
+  # Group the data by sim_number and add columns for x and y
+  df[, y := actuar::rparalogis(n = .N, shape = shape, rate = rate, scale = scale)]
+
+  # Compute the density of the y values and add columns for dx and dy
+  df[, c("dx", "dy") := density(y, n = n)[c("x", "y")], by = sim_number]
+
+  # Compute the p-values for the y values and add a column for p
+  df[, p := actuar::pparalogis(y, shape = shape, rate = rate, scale = scale)]
+
+  # Compute the q-values for the p-values and add a column for q
+  df[, q := actuar::qparalogis(p, shape = shape, rate = rate, scale = scale)]
+
+  if(.return_tibble){
+    df <- dplyr::as_tibble(df)
+  } else {
+    data.table::setkey(df, NULL)
+  }
+
+  # Create a tibble of the parameter grid
   param_grid <- dplyr::tibble(.shape, .rate, .scale)
 
   # Attach descriptive attributes to tibble
@@ -116,6 +123,7 @@ tidy_paralogistic <- function(.n = 50, .shape = 1, .rate = 1, .scale = 1 / .rate
   attr(df, ".scale") <- .scale
   attr(df, ".n") <- .n
   attr(df, ".num_sims") <- .num_sims
+  attr(df, ".ret_tbl") <- .return_tibble
   attr(df, "tibble_type") <- "tidy_paralogistic"
   attr(df, "ps") <- ps
   attr(df, "qs") <- qs
